/*** COPYRIGHT SECTION *******************************************************
* ( c ) COPYRIGHT 2007 - KONTRON CANADA INC. All rights reserved
* This software is confidential and proprietary to KONTRON CANADA
* KONTRON CANADA INC.
* 616 Cur�Boivin st., Boisbriand, CANADA
* J7G-2A7
*****************************************************************************
*** DESCRIPTION SECTION *****************************************************
* File Name       :  aasniff.h
* Creation Date   :  15-june-2007
* Programmer      :  Chanthy Toeung
* Description     :  this is the main module of the program. It open the 
*                    devices and start sniffing.
****************************************************************************/
/*==========================================================================
* DOC
* ==========================================================================
* INSTALLATION  
*  1. Install aardvark CD
*  2. Go to the www.totalphase.com to update the software and firmware of 
*    aardvark device to version 4.0
*  3. execute by :  aasniff     pFileName   port_numberA   ( port_numberB )
* ==========================================================================
* INCLUDES
* =========================================================================*/
#include <stdio.h>
#include <stdlib.h>

#include "aardvark.h"
#include "sniff.h"
#include "ipmipacket.h"
#include "hexdump.h"
#include "display.h"
#include "cvt2pcap.h"

/*=========================================================================
* CONSTANTS
* =======================================================================*/


/*=========================================================================
* MAIN PROGRAM
* ========================================================================*/
int main ( int argc, char *argv[] ) {
   int argcUsed = 0;
   int result;
   int i;
   char* pFileName;
   char* pWriteFileName;
   int fin;
   int portAmount;
   unsigned char pcapFileName[50]="out";
   fin = FALSE;
   if ( argc < 2 ) {      
      printf( "Invalid command.\n\n" );
      usage();      
      return 1;
   }
   else{
      normalProcess = get_option( argc, argv );
      quiet = get_quiet_option();
      debug = get_debug_level();
      withOption = get_opt();
   }
   if( normalProcess ){
      pFileName = "rawData";
      /* without option, the second argument will be the filename */
      if( !withOption ){
         pWriteFileName = argv[1];
         argcUsed = 2;  /* 2 arguments have been used */
      }
      else{
         if( quiet ){
            pWriteFileName = argv[2];            
            argcUsed = 3; /* 3 arguments have been used */
         }
         else{
            pWriteFileName = argv[3];
            argcUsed = 4; /* 4 arguments have been used */
         }
      }
      portAmount = ( argc - argcUsed ); /* determine the number of port */
      if( portAmount <= 0 ){
         printf("No Aardvark port has been detected\n");
         return 1;
      }
      /* declare the Aardvark variable according to the number of port finding*/
      aa_u16 devices[2];
      int trouve = 0;
      int port[portAmount];
      Aardvark handle[portAmount];
      AardvarkExt aaext[portAmount];
      trouve = aa_find_devices( 5, devices );
	printf("Number of devices found = %d\n", trouve);
	printf("port: %04x\n", devices[0]);
	printf("port: %04x\n", devices[1]);
      for(  i = 0; i < portAmount; i++  ){
         port[i]    = atoi( argv[i+ argcUsed] );
         handle[i]  = aa_open_ext( devices[i], &aaext[i] );
         if ( handle[i] <= 0 ) {
            printf( "Unable to open Aardvark device on port %d\n", port[i] );
            printf( "Error code = %d\n", handle[i] );
            return 1;
         }
         if( !quiet )
            printf( "Opened Aardvark; features = 0x%02x\n", aaext[i].features );
         /* Disable the I2C bus pullup resistors ( 2.2k resistors ).
         * This command is only effective on v2.0 hardware or greater.
         * The pullup resistors on the v1.02 hardware are enabled by default. */
         aa_i2c_pullup( handle[i], AA_I2C_PULLUP_NONE );
         /* Disable the Aardvark adapter's power pins.
         * This command is only effective on v2.0 hardware or greater.
         * The power pins on the v1.02 hardware are not enabled by default. */
         aa_target_power( handle[i], AA_TARGET_POWER_NONE );
         /* Enable the monitor*/
         result = aa_i2c_monitor_enable( handle[i] );
	 sleep(0.1);
         if ( result < 0 ) {
            printf( "error: %s\n", aa_status_string( result ) );
            return 1;
         }
         if( !quiet )
            printf( "Enabled I2C monitor.\n" );
      }
      /* Start dumping raw data */
      dumpSniff ( handle, pFileName, portAmount );
      /* Disable the monitor and close the device */
      for( i = 0; i < portAmount; i++ ){
         aa_i2c_monitor_disable( handle[i] );
         sleep(0.1);
         aa_close( handle[i] );
         sleep(0.1);
      }
      /* build ipmb packet */
      fipmipacket(  pFileName );
      /* build ipmb matching packet */
      fmatching_packet( "matching.txt" );
      /* create hexdump file of the receiving packets */
      hexdump_file( "matching.txt", pWriteFileName ); 
      if(strlen(pWriteFileName) < 50)
      {  /* Keep a copy of the file name for pcap */
         /*strcpy(pcapFileName, pWriteFileName);*/
      }
      /* convert from hexdump file to pcap file */
      /*cvt2pcap(pWriteFileName,pcapFileName );*/
   }
   return 0;
}
