/*** COPYRIGHT SECTION ****************************************************
* ( c ) COPYRIGHT 2007 - KONTRON CANADA INC. All rights reserved
* This software is confidential and proprietary to KONTRON CANADA
* KONTRON CANADA INC.
* 616 Cur�Boivin st., Boisbriand, CANADA
* J7G-2A7
***************************************************************************
*** DESCRIPTION SECTION ***************************************************
* File Name       :  ipmipacket.c
* Creation Date   :  15-june-2007
* Modify Date     :  11-July-2007
* Programmer      :  Chanthy Toeung
* Description     :  The module that contain the structure and the function
*                    to build ipmi packet 
**************************************************************************/
/**=========================================================================
* INCLUDES
*=========================================================================*/
#include "ipmipacket.h"
#include <string.h>
#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include "display.h"

/**************************************************************************
*
* Description  :  create the ipmi packet order by time and stored them in 
*                 "Request" or "Response" file respectively for each valid 
*                 request and response found in the raw data.
*
* Restriction  : 
*
* Input        :  a pointer via a file name that contain raw data
*                 a pointer via a file name that will store the new ipmi packet
*
* Output       :  provide the following files output
*                 Request : stored request command as binary file
*                 Response : stored response command as binary file
*                 Req.txt and Res.txt : text file of request and reponse command
*
* Global       :  the format of the file will be :
*                 ipmi header format ipmi body format ipmi body format ....
*        
* Return       :  non
*
***************************************************************************/
void fipmipacket (  char *pFileName ){
   FILE*   pFileRead; /*File stream for reading rawdata file*/
   FILE*   pFileRq;  /*File stream for writing Request command*/
   FILE*   pFileRs; /*File strean for writing Response command*/
   int     i, l, k;
   int     rq = 0; /* counter for request */
   int     rs = 0;  /*counter for response */
   u16     data[DATA_SIZE];
   u08     netFn;
   tIpmb_Header   rqIpmbHeader;
   tIpmb_Header   rsIpmbHeader;
   tIpmbPacket    rqPaquet;
   tIpmbPacket    rsPaquet;   
   struct timeval rqTime;
   /*get option*/
   quiet = get_quiet_option();
   debug = get_debug_level();
   withOption = get_opt();
   normalProcess = TRUE;
   pFileRead = fopen( pFileName, "r" );

   if ( pFileRead == NULL && ( !quiet ) ){
      printf( "error can't open the file name : %s\n", pFileName );
      return;
   }   
   else{
      pFileRq = fopen( "Request", "w" );
      pFileRs = fopen( "Response", "w" );

      if ( pFileRq == NULL || pFileRs == NULL ){
         if( !quiet )
            printf( "error can't open Request or Response file\n" );
      }
      if ( !quiet ){
         printf( "start creating request and response files\n" );
      }
      i = fread(  &rqTime, sizeof( rqTime ), 1, pFileRead  );         
      i = fread(  &data, sizeof( data ), 1, pFileRead  );         
      while( !feof( pFileRead ) ){ 
         /*interpret data into ipmi format*/
	      if( data[0] > 7){
            netFn = data[3] >> 2;       
            /*this is a request*/
            if ( (netFn % 2) == 0 ){            
               rqPaquet.waitTime = rqTime;
               /*header of request format*/
               rqIpmbHeader.SlaveAdd = data[2];   /* responder slave addresse*/
               rqIpmbHeader.netFn = netFn;
               rqIpmbHeader.LUN = ( data[3] & 0x03 );   /* respond LUN */
               rqIpmbHeader.checkSum = data[4];
               /*body of the request format*/
               rqPaquet.SlaveAdd = data[5];   /* requester slave addresse */               
               rqPaquet.rqSeq = data[6] >> 2 ;
               rqPaquet.LUN = ( data[6] & 0x03 ) ;   /* request LUN */
               rqPaquet.cmd = data[7];
               rqPaquet.data[0] = data[0]; /* message size */
   			   
               for ( l = 8; l < data[0]; l++ ){
                  rqPaquet.data[l-7] = data[l];
               }
               k = fwrite( &rqIpmbHeader, sizeof(rqIpmbHeader), 1, pFileRq );
               if( k == 0 ){
                  printf("request file err1 = %d\n", ferror( pFileRq ) );
                  fclose( pFileRq );
                  pFileRq = fopen( "Request", "a" );
               }   
               k = fwrite( &rqPaquet, sizeof( rqPaquet ), 1, pFileRq );
               if( k == 0 ){
                  printf("request file err2 = %d\n", ferror( pFileRq ) );
                  fclose( pFileRq );
                  pFileRq = fopen( "Request", "a" );
               }   
               rq++;
            }
            /*this is a reply*/
            else{            
               rsPaquet.waitTime = rqTime;
               /*header of respond format*/
               rsIpmbHeader.SlaveAdd = data[2];
               rsIpmbHeader.netFn = netFn;
               rsIpmbHeader.LUN = ( data[3] & 0x03 ) ;            
               rsIpmbHeader.checkSum = data[4];
               /*body of the respond format*/
               rsPaquet.SlaveAdd = data[5];
               rsPaquet.rqSeq = data[6] >> 2 ;            
               rsPaquet.LUN = ( data[6] & 0x03 ) ;            
               rsPaquet.cmd = data[7];
               rsPaquet.data[0] = data[0]; /*message/data size*/
   			   
               for ( l = 8; l < data[0]; l++ ){
                  rsPaquet.data[l-7] = data[l];
               }
               k = fwrite( &rsIpmbHeader, sizeof(rsIpmbHeader), 1, pFileRs );
               if( k == 0 ){
                  printf("response file err1 = %d\n", ferror( pFileRs ) );
                  fclose( pFileRs );
                  pFileRs = fopen( "Response", "a" );
               }
               k = fwrite( &rsPaquet, sizeof( rsPaquet ), 1, pFileRs );
               if( k == 0 ){
                  printf("response file err2 = %d\n", ferror( pFileRs ) );
                  fclose( pFileRs );
                  pFileRs = fopen( "Response", "a" );
               } rs++;
            }
	      }
	      else if( data[0]>2 ){
   	      if( !quiet ){
   	         if( debug > DEBUG_LEVEL_RAW ){
   	            printf("Invalid packet; this packet has only %d byte(s)\n",
   	                  data[0]);
	            }
	         }

	         switch( data[0] ){
   	         case 4:
   	            rqPaquet.waitTime = rqTime;
                  rqIpmbHeader.SlaveAdd = data[2];   /* responder slave addresse*/
                  rqPaquet.data[0] = data[0]; /*message/data size*/
                  break;
   	         case 5:
   	            rqIpmbHeader.netFn = netFn;
                  rqIpmbHeader.LUN = ( data[3] & 0x03 );   /* respond LUN */
                  break;
   	         case 6:
   	            rqIpmbHeader.netFn = netFn;
                  rqIpmbHeader.LUN = ( data[3] & 0x03 );   /* respond LUN */
                  rqIpmbHeader.checkSum = data[4];   	         
                  break;
   	         case 7:
   	            rqIpmbHeader.netFn = netFn;
                  rqIpmbHeader.LUN = ( data[3] & 0x03 );   /* respond LUN */
                  rqIpmbHeader.checkSum = data[4];   	         
                  rqPaquet.SlaveAdd = data[5];   /* requester slave addresse */
                  break;
               default:
                  break;
	         }
	         k = fwrite( &rqIpmbHeader, sizeof(rqIpmbHeader), 1, pFileRq );
            if( k == 0 ){
               printf("request file err1 = %d\n", ferror( pFileRq ) );
               fclose( pFileRq );
               pFileRq = fopen( "Request", "a" );
            }   
            k = fwrite( &rqPaquet, sizeof( rqPaquet ), 1, pFileRq );
            if( k == 0 ){
               printf("request file err2 = %d\n", ferror( pFileRq ) );
               fclose( pFileRq );
               pFileRq = fopen( "Request", "a" );
            }   
            rq++;
	      }
	      else{
   	      /*there is no more data, only start and stop byte is in the data*/
   	      printf("[S]  [p]\n");
	      }
         i = fread(  &rqTime, sizeof( rqTime ), 1, pFileRead  );         
         i = fread(  &data, sizeof( data ), 1, pFileRead  );               
      }
      if( !quiet )
         printf( "End creating Request and Response files\n" );
      fclose( pFileRq );
      fclose( pFileRs );
   }
   fclose( pFileRead );
   
}
/******************************************************************************
*
* Description  :  Matching the request to the correspondant response and store 
*                 it in a file whose name is given by user
*
* Restriction  : 
*
* Input        :  a pointer via a file name that will store the new ipmi packet
*
* Output       :  provide a file that contain ipmimatching packet
*
* Global       :  the format of the file will be :
*              Request command 001
*              Response command 001
*              Request command 002
*              Response command 002
*              ....
* Return       : non
*
***************************************************************************/
void fmatching_packet( char* pWriteFileName )
{
   FILE*     pFileRq;
   FILE*     pFileRs;
   FILE*     pFileWrite;
   int       match;
   int       i, j, k;
   double    allTime;
   double    diffTime;   /*actual different time between request and response*/
   tIpmb_Header   rqIpmbHeader;
   tIpmb_Header   rsIpmbHeader;
   tIpmbPacket    rqPaquet;
   tIpmbPacket    rsPaquet;
   long currentPosFileRs, lastPosFileRs;
   int pk = 0;  
   int count;
   int skip = MATCHED;
   int invalidPacket;
   long lastPacketPos;
   invalidPacket = FALSE;
   match = UN_MATCHED; /*Comparison condition for matching packet*/
   quiet = get_quiet_option();
   debug = get_debug_level();
   
   pFileRq = fopen( "Request", "r" );
   pFileRs = fopen( "Response", "r" );
   if ( pFileRq == NULL || pFileRs == NULL ){
      if( !quiet )
         printf( "error can't open Request or Response file\n" );
      return;
   }
   else{
      if( !quiet )
         printf( "Please wait while matching packets is in process.\n" );
      printf( "This process may take several minutes....\n" );
      pFileWrite = fopen( pWriteFileName, "w" );

      /* Read the first request from file */
      i = fread(  &rqIpmbHeader, sizeof( rqIpmbHeader ), 1, pFileRq );         
      i = fread(  &rqPaquet, sizeof( rqPaquet ), 1, pFileRq  );
      count = 1;
      /* Read the first response from file */
      j = fread(  &rsIpmbHeader, sizeof( rsIpmbHeader ), 1, pFileRs );         
      j = fread(  &rsPaquet, sizeof( rsPaquet ), 1, pFileRs  );      
      /*When we reach the end of the file of Request file, we will abort the 
        matching mission */   
      while( !feof( pFileRq ) ){         
         if( !quiet ){
            if( debug > DEBUG_LEVEL_RAW )
               printf("Read packet number %d from file \n", count);
         }
         if( rqPaquet.data[0] < 7 ){
            /*Write the request to file even there is no response is matched*/
            k = fwrite( &rqIpmbHeader, sizeof( rqIpmbHeader ),1, pFileWrite );   
            k = fwrite( &rqPaquet, sizeof( rqPaquet ), 1, pFileWrite );
            pk++;
            match = MATCHED;          
            invalidPacket = TRUE;
         }
         while ( match == UN_MATCHED && !feof(pFileRs) ){
            invalidPacket = FALSE;          
            allTime = ( double )rqPaquet.waitTime.tv_sec + \
                     ( double )( rqPaquet.waitTime.tv_usec/1000000.0 );
            diffTime = ( double )rsPaquet.waitTime.tv_sec + \
                     ( double )( rsPaquet.waitTime.tv_usec/1000000.0 );
            diffTime = diffTime - allTime;            
            if( (diffTime < 0) && abs(diffTime) > DELAY ){
               /* read the next response from file */
               j = fread( &rsIpmbHeader, sizeof( rsIpmbHeader ), 1, pFileRs );         
               j = fread( &rsPaquet, sizeof( rsPaquet ), 1, pFileRs  );
            }
            else if ( diffTime > RQ_TO_RS_TIME ){
               if( !quiet ){
                  if( debug > DEBUG_LEVEL_RAW )
                     printf( "Time out; Abort packet %d\n", count );
               }
               rewind( pFileRs );
               match = MATCHED;
               /* Write the request to file even no response is matched */
               k = fwrite( &rqIpmbHeader, sizeof( rqIpmbHeader ),1, pFileWrite );   
               k = fwrite( &rqPaquet, sizeof( rqPaquet ), 1, pFileWrite );
               pk++;
               break;
            }
            else{               
               if( ( rsPaquet.LUN == rqIpmbHeader.LUN ) && \
                     ( rsPaquet.rqSeq == rqPaquet.rqSeq ) ){            
                  if( ( rsIpmbHeader.netFn - rqIpmbHeader.netFn ) == 1 &&
                         ( rsPaquet.cmd == rqPaquet.cmd ) ){             
                     if( ( rqPaquet.SlaveAdd == rsIpmbHeader.SlaveAdd ) && 
                            ( rsPaquet.SlaveAdd == rqIpmbHeader.SlaveAdd ) ){            
                        if( abs(diffTime) <= RQ_TO_RS_TIME ){
                           /* write the matching pair to the file */
                           if( pFileWrite != NULL ){
                              k = fwrite( &rqIpmbHeader, sizeof( rqIpmbHeader ),
                                          1, pFileWrite );   
                              k = fwrite( &rqPaquet, sizeof( rqPaquet ),
                                          1, pFileWrite );
                              k = fwrite( &rsIpmbHeader, sizeof( rsIpmbHeader ),
                                          1, pFileWrite );    
                              k = fwrite( &rsPaquet, sizeof( rsPaquet ),
                                          1, pFileWrite );                           
                              match = MATCHED ;
                              pk += 2;
                              lastPacketPos = ftell( pFileRs );
                              if( skip == UN_MATCHED ){
                                 rewind( pFileRs );
                                 skip = MATCHED;
                              }
                              if( !quiet ){
                                 if( debug > DEBUG_LEVEL_RAW )
                                    printf( "matched pck %d found.\n", count );
                              }
                           }
                           else{
                              if( !quiet ){
                                 if( debug > DEBUG_LEVEL_RAW )
                                   printf("Unable to write packets to file.\n");
                              }
                           }                                                      
                        }
                        else{ /* Time out */
                           match = MATCHED;
                           /*Set the position of file response to the previous 
                           * position */
                           if( !quiet ){
                              if( debug > DEBUG_LEVEL_RAW )
                                 printf( "Over time, no matching found.\n" );
                           }
                        }
                     }
                     else{
                        /* Read the next response from file */
                        j = fread( &rsIpmbHeader, sizeof( rsIpmbHeader ),
                                     1, pFileRs );         
                        j = fread( &rsPaquet, sizeof( rsPaquet ), 1, pFileRs );
                        skip = UN_MATCHED; 
                        if( !quiet ){
                           if( debug > DEBUG_LEVEL_RAW )
                              printf( "1 No matching found in case .\n" );
                        }
                     }
                  }
                  else{
                     /* Read the next response from file */
                     j = fread( &rsIpmbHeader, sizeof( rsIpmbHeader ),
                                 1, pFileRs );         
                     j = fread( &rsPaquet, sizeof( rsPaquet ), 1, pFileRs );
                     skip = UN_MATCHED; 
                     if( !quiet ){
                        if( debug > DEBUG_LEVEL_RAW )
                           printf( "2 No matching found.\n" );
                     }
                  }
               }
               else{ /*differet request seq or lun*/
                  /* Read the next response from file */
                  lastPosFileRs = ftell( pFileRs );
                  j = fread( &rsIpmbHeader, sizeof( rsIpmbHeader ), 1, pFileRs);         
                  j = fread( &rsPaquet, sizeof( rsPaquet ), 1, pFileRs ); 
                  currentPosFileRs = ftell( pFileRs );
                  skip = UN_MATCHED;
                  if( !quiet ){
                     if( debug > DEBUG_LEVEL_RAW )
                        printf( "3 No matching found.\n" );
                  }
               }
            }
         }  /* end of comparison loop */
         /* write the non matching request to file even no response is found */
         if( match == UN_MATCHED && feof(pFileRs) ){
            k = fwrite( &rqIpmbHeader, sizeof( rqIpmbHeader ),1, pFileWrite );   
            k = fwrite( &rqPaquet, sizeof( rqPaquet ), 1, pFileWrite );
            pk++;
         }         
         /* Read the next request from file */
         if( pFileRq != NULL ){
            i = fread( &rqIpmbHeader, sizeof( rqIpmbHeader ), 1, pFileRq );         
            i = fread( &rqPaquet, sizeof( rqPaquet ), 1, pFileRq  );
            if( ferror( pFileRq ) ){
               printf("error in matching rq\n");
               perror("err = ");
               break;
             }
            count++;
         }
         /* Read the next response from file */
         if( !feof(pFileRs) && (!invalidPacket) ){
            j = fread( &rsIpmbHeader, sizeof( rsIpmbHeader ), 1, pFileRs );         
            j = fread( &rsPaquet, sizeof( rsPaquet ), 1, pFileRs  );         
         }
         match = UN_MATCHED;  /*Reset the condition for comparison*/
      }
      /* Record the unmatching responses to file */
      rewind( pFileRs );
      fseek( pFileRs, lastPacketPos, SEEK_SET );
      j = fread( &rsIpmbHeader, sizeof( rsIpmbHeader ), 1, pFileRs );         
      j = fread( &rsPaquet, sizeof( rsPaquet ), 1, pFileRs  );
      while( !ferror( pFileRs ) && !feof( pFileRs ) ){
         k = fwrite( &rsIpmbHeader, sizeof( rsIpmbHeader ), 1, pFileWrite );    
         k = fwrite( &rsPaquet, sizeof( rsPaquet ), 1, pFileWrite );  
         j = fread( &rsIpmbHeader, sizeof( rsIpmbHeader ), 1, pFileRs );         
         j = fread( &rsPaquet, sizeof( rsPaquet ), 1, pFileRs  );
         pk++;
      } 
   }
   if( !quiet ){
      printf( "number of packet write to file %s  %d\n", pWriteFileName, pk );
   }
   printf( "number of packet write to file %s  %d\n", pWriteFileName, pk );
   fclose( pFileWrite );
   fclose( pFileRq );
   fclose( pFileRs );
}

