/*** COPYRIGHT SECTION *****************************************************
* (c) COPYRIGHT 2007 - KONTRON CANADA INC. All rights reserved
* This software is confidential and proprietary to KONTRON CANADA
* KONTRON CANADA INC.
* 616 Cur�Boivin st., Boisbriand, CANADA
* J7G-2A7
****************************************************************************
*** DESCRIPTION SECTION ****************************************************
* File Name       :  ipmipacket.h
* Creation Date   :  19-june-2007
* Programmer      :  Chanthy Toeung
* Description     :  the file containing the structure of ipmi packet  
*                    and its functions
***************************************************************************/
#ifndef IPMIPACKET_H
#define IPMIPACKET_H
/*=========================================================================
* INCLUDES
*=========================================================================*/
#include "aardvark.h"
#include <sys/time.h>
/*=========================================================================
* CONSTANTS
*=========================================================================*/
#define DATA_SIZE        64

/* maximum waiting time to get the reply after a request is made
*  RQ_TO_RS_TIME ~= 2* TIME_MAX = 700 milliseconds in specification IPMI
*  Read the specifications IPMI v1.0 table 4-1 for more detail
*/
#define DELAY                 0.4  /* 400 milli seconds of delay */
#define RQ_TO_RS_TIME         0.7  /* 700 milli seconds */
#define MATCHED               0    /* matching packet */
#define UN_MATCHED            1    /* non matching packet */
#define INVALID_PACKET_SIZE   100
/*=========================================================================
| GENERAL TYPE DEFINITIONS
 ========================================================================*/
 
/*=========================================================================
*   Packet HEADER of ipmb packet
*=========================================================================*/
struct tIpmb_Header{
   u08 SlaveAdd;     /* slave adresse */
   u08 netFn;       /* NetFn */
   u08 LUN;        /* this can be rqLUN or rsLUN depending on NetFn */
   u08 checkSum;  /* header checksum */
};

typedef struct tIpmb_Header tIpmb_Header;

/*=========================================================================
*   STRUCTURE DE DONN�S DU PAQUET IPMB
*=========================================================================*/
struct tIpmbPacket{
   struct timeval waitTime;
   u08 SlaveAdd;   
   u08 rqSeq;     /* Request sequence */
   u08 LUN;
   u08 cmd;       /* command */
   u08 data[DATA_SIZE]; /* data */  
};

typedef struct tIpmbPacket tIpmbPacket;
/*=========================================================================
* FUNCTIONS
*=========================================================================*/

void fipmipacket(
   char* pFileName
);

void fmatching_packet(
   char* pWriteFileName
);

#endif
