/*** COPYRIGHT SECTION *****************************************************
* ( c ) COPYRIGHT 2007 - KONTRON CANADA INC. All rights reserved
* This software is confidential and proprietary to KONTRON CANADA
* KONTRON CANADA INC.
* 616 Cur�Boivin st., Boisbriand, CANADA
* J7G-2A7
****************************************************************************
*** DESCRIPTION SECTION ****************************************************
* pFile Name      :  sniff.c
* Creation Date   :  15-june-2007
* Programmer      :  Chanthy Toeung
* Description     :  The sniffer module that containt the data structure to
*                 communicate with Aardvark devices
***************************************************************************/
/*=========================================================================
* INCLUDES
*=========================================================================*/
#include "sniff.h"
#include "display.h"

static u16   data[BUFFER_SIZE];
static struct termios termattr, save_termattr;
static int ttysavefd = -1;

/*=========================================================================
* TYPEDEFS
*=========================================================================*/

static enum 
{ 
  RESET, RAW, CBREAK 
} ttystate = RESET;

/****************************************************************************
*
* Description  :  sniff the i2c data in mode dump
*
* Restriction  :  it can create 1 ipmb packet file only
*
* Input        :  - Aardvark handle
*                 - hadleAmount : Number of Aardvark device that are using
*
* Output       :  - filename    : name of the file that contain i2c raw data
*
* Global       :  the following code is from the aamonitor.c of aardvark example
*              with modification.
*              for more detail visit : www.totalphase.com
*here is the format of the data that has been store in the  file
**--------*-----*----------*----------*-------*----------*--------*----*-------*
*|Msg size| Date| rq/rs Add| netfn/LUN| chsum1| rq/rs Add| Seq/LUN| cmd| chsum2|
**--------*-----*----------*----------*-------*----------*--------*----*-------*
*
* Return       : non
*
*******************************************************************************/
void dumpSniff ( Aardvark handle[MAX_AARDVARK_DEVICE],
                 char* pFileName, int handleAmount )
{
   int result[handleAmount];
   u16 last_data1 = 0;
   u16 last_data0 = 0;
   FILE* pFile;
   u16 dataWrite[MESSAGE_SIZE];
   struct timeval res;
   int i, j, k;
   int boucle = 0;
   int num_bytes = 0;
   unsigned char ch; 

   /* initialise all static variable of diplay.h to advoid warning */
   debug = get_debug_level();
   normalProcess = TRUE;
   withOption = get_opt();
   quiet = get_quiet_option();
   if( handleAmount > MAX_AARDVARK_DEVICE ){
      printf("Aardvark devices Limits exceeded!\n");
      return;
   }
   /* Empty the pFile b4 writing new entries */
   pFile = fopen( pFileName, "w" );   
   for( j = 0; j < handleAmount; j++ ){      
      if( !quiet && ( debug != DEBUG_LEVEL_PACKET ) ){
         printf( "Waiting %d ms for first transaction...\r\n", INITIAL_TIMEOUT );
      }
      result[j] = aa_async_poll( handle[j], INITIAL_TIMEOUT );
      if ( result[j] == AA_ASYNC_NO_DATA ) {
         if( !quiet && ( debug < 2 ) ){
            printf( "  no data pending on port %d .\r\n", j );
         }
         return;
      }
      if( !quiet && ( debug != DEBUG_LEVEL_PACKET ) ){    
         printf( "  data received.\r\n" );
      }
   }
   set_tty_raw(); /* set up character-at-a-time */
   /* Loop until aa_async_poll times out */
   for ( ;; ) {
       /* Read the next monitor transaction.
       * This function has an internal timeout ( see datasheet ), though
       * since we have already checked for data using aa_async_poll,
       * the timeout should never be exercised.
       */
	   for( j = 0; j < handleAmount; j++ ){
   	   boucle++;
   	   printf("boucle = %d     port = %d\r\n", boucle, aa_port(handle[j]));
   	   set_tty_cooked();
   	   if( result[j] == AA_ASYNC_I2C_MONITOR ){
            num_bytes = aa_i2c_monitor_read( handle[j], BUFFER_SIZE, data );
         }
         else
            num_bytes = AA_OK;
         set_tty_raw(); /* set up character-at-a-time */            
         if ( num_bytes < 0 ) {
            printf( "error: %s\r\n", aa_status_string( num_bytes ) );
            return;
         }
         for ( i = 0; i < num_bytes; ++i ) {
            /* the first byte of dataWrite indicate 
            *  the total number of byte in the rq or rs packet */
            dataWrite[0] = num_bytes + 1;
            /*Check for START condition*/
            if ( data[i] == AA_I2C_MONITOR_CMD_START ) {           
      	      gettimeofday( &res, NULL );
               if( !quiet && ( debug != DEBUG_LEVEL_PACKET ) ){		  
                  printf(  "\r\n[S]"  );
               }
            } 
            /* Check for STOP condition */
            else if ( data[i] == AA_I2C_MONITOR_CMD_STOP ) {
               if( !quiet && ( debug != DEBUG_LEVEL_PACKET ) ){
                  printf( "[P]\r\n" );
               }
            }
		      /* check for the slave address */
            else {
               int nack = ( data[i] & AA_I2C_MONITOR_NACK );
               /* 7-bit addresses */
               if ( last_data0 == AA_I2C_MONITOR_CMD_START &&
                     ( ( data[i] & 0xf9 ) != 0xf0 || nack ) ) {  
                  if( !quiet && ( debug != DEBUG_LEVEL_PACKET ) ){
                     printf( "<%02x:%c>%s ", ( data[i] & 0xff ) >> 1,
                           data[i] & 0x01 ? 'r' : 'w', nack ? "*" : "" );
                  }
                  dataWrite[i+1] = data[i];
               }
               /* 10-bit addresses
               * See Philips specification for more details.
	       */
               else if ( last_data1 == AA_I2C_MONITOR_CMD_START &&
                       ( last_data0 & 0xf9 ) == 0xf0 ) {
                  if( !quiet && ( debug != DEBUG_LEVEL_PACKET ) ){
                     printf( "<%03x:%c>%s ",
                      ( ( last_data0 << 7 ) & 0x300 ) | ( data[i] & 0xff ),
                          last_data0 & 0x01 ? 'r' : 'w', nack ? "*" : "" );
                  }
                  dataWrite[i+1] = data[i];
               }
               /* Normal data*/
               else if ( last_data0 != AA_I2C_MONITOR_CMD_START ) {
                  if( !quiet && ( debug != DEBUG_LEVEL_PACKET ) ){
                     printf( "%02x%s ", data[i] & 0xff, nack ? "*" : "" );
                  }
                  dataWrite[i+1] = data[i];
               }
            }
            last_data1 = last_data0;
            last_data0 = data[i];
         }/* end of " for numbyte " loop */
         dataWrite[i] = j; /*register port number*/
         /* assign all bytes that haven't use to 0 */
         for ( i = num_bytes + 2 ; i< MESSAGE_SIZE -1; i++ ){
            dataWrite[i] = 0x00;
	      }
         /* start writing to File here */
         if( pFile != NULL && num_bytes > 0 ){
            k = fwrite( &res, sizeof( res ), 1, pFile );   
            k = fwrite( &dataWrite, sizeof(dataWrite), 1, pFile );
         } /* end of writing to File */
         if( !quiet && ( debug != DEBUG_LEVEL_PACKET ) ){
            printf( "\r\nWaiting %d ms for subsequent transaction...\r\n",
                 INTERVAL_TIMEOUT );
         }
         result[j] = aa_async_poll( handle[j], INTERVAL_TIMEOUT );
         if ( result[j] == AA_ASYNC_NO_DATA ) {
            if( !quiet && ( debug != DEBUG_LEVEL_PACKET ) )
               printf( "  no more data pending on port %d .\r\n", j );
            num_bytes = 0;
         }
   	   else{
      	   if( !quiet && ( debug != DEBUG_LEVEL_PACKET ) )
               printf( "data received on port %d .\r\n", j );
         }
         /* waiting for keyborad interrupt */

         ch = kb_getc();      /* char typed by user?*/
         if ( ch == 0x03 ){      /* might be control-C*/
            set_tty_cooked();  /* control-C, restore normal TTY mode*/ 
            fclose( pFile );
     	      if( !quiet && ( debug >= DEBUG_DEFAULT ) )
               printf("FINISH WRITING RAW DATA INTO FILE NAME rawData\r\n\n");
            return ; 
         }
         fflush( stdout );
	   }
   }
   set_tty_cooked();
}

/* ***************************************************************************
 *
 * set_tty_raw(), put the user's TTY in one-character-at-a-time mode.
 * returns 0 on success, -1 on failure.
 *
 *************************************************************************** */
int
set_tty_raw(void) 
{
  int i;

  i = tcgetattr (STDIN_FILENO, &termattr);
  if (i < 0) 
  {
    printf("tcgetattr() returned %d for fildes=%d\n",i,STDIN_FILENO); 
    perror ("");
    return -1;
  }
  save_termattr = termattr;

  termattr.c_lflag &= ~(ECHO | ICANON | IEXTEN | ISIG);
  termattr.c_iflag &= ~(BRKINT | ICRNL | INPCK | ISTRIP | IXON);
  termattr.c_cflag &= ~(CSIZE | PARENB);
  termattr.c_cflag |= CS8;
  termattr.c_oflag &= ~(OPOST);
   
  termattr.c_cc[VMIN] = 1;  /* or 0 for some Unices;  see note 1 */
  termattr.c_cc[VTIME] = 0;

  i = tcsetattr (STDIN_FILENO, TCSANOW, &termattr);
  if (i < 0) 
  {
    printf("tcsetattr() returned %d for fildes=%d\n",i,STDIN_FILENO); 
    perror("");
    return -1;
  }
   
  ttystate = RAW;
  ttysavefd = STDIN_FILENO;

  return 0;
}

/* ***************************************************************************
 *
 * set_tty_cbreak(), put the user's TTY in cbreak mode.
 * returns 0 on success, -1 on failure.
 *
 *************************************************************************** */
int 
set_tty_cbreak() 
{
  int i;

  i = tcgetattr (STDIN_FILENO, &termattr);
  if (i < 0) 
  {
    printf("tcgetattr() returned %d for fildes=%d\n",i,STDIN_FILENO); 
    perror ("");
    return -1;
  }

  save_termattr = termattr;

  termattr.c_lflag &= ~(ECHO | ICANON);
  termattr.c_cc[VMIN] = 1;
  termattr.c_cc[VTIME] = 0;
      
  i = tcsetattr (STDIN_FILENO, TCSANOW, &termattr);
  if (i < 0) 
  {
    printf("tcsetattr() returned %d for fildes=%d\n",i,STDIN_FILENO); 
    perror ("");
    return -1;
  }
  ttystate = CBREAK;
  ttysavefd = STDIN_FILENO;

  return 0;
}

/* ***************************************************************************
 *
 * set_tty_cooked(), restore normal TTY mode. Very important to call
 *   the function before exiting else the TTY won't be too usable.
 * returns 0 on success, -1 on failure.
 *
 *************************************************************************** */
int
set_tty_cooked() 
{
  int i;
  if (ttystate != CBREAK && ttystate != RAW) 
  {
    return 0;
  }
  i = tcsetattr (STDIN_FILENO, TCSAFLUSH, &save_termattr);
  if (i < 0) 
  {
    return -1;
  }
  ttystate = RESET;
  return 0;
}

/* ***************************************************************************
 *
 * kb_getc(), if there's a typed character waiting to be read,
 *   return it; else return 0.
 * 10-sep-2006: kb_getc() fails (it hangs on the read() and never returns
 * until a char is typed) under some Unix/Linux versions: ubuntu, suse, and
 * maybe others. To make it work, please uncomment two source lines below.
 *
 *************************************************************************** */
unsigned char
kb_getc(void) 
{
  int i;
  unsigned char ch;
  ssize_t size;
  termattr.c_cc[VMIN] = 0;  /* uncomment if needed */
  i = tcsetattr (STDIN_FILENO, TCSANOW, &termattr);
  size = read (STDIN_FILENO, &ch, 1);
  termattr.c_cc[VMIN] = 1;  /* uncomment if needed */
  i = tcsetattr (STDIN_FILENO, TCSANOW, &termattr);
  if (size == 0)
  {
    return 0;
  }
  else
  {
    return ch;
  }
}

/* ***************************************************************************
 *
 * kb_getc_w(), wait for a character to be typed and return it.
 *
 *************************************************************************** */
unsigned char
kb_getc_w(void) 
{
  unsigned char ch;
  size_t size;

  while (1)
  {

    usleep(20000);        /* 1/50th second: thanks, Floyd! */

    size = read (STDIN_FILENO, &ch, 1);
    if (size > 0)
    {
      break;
    }
  }
  return ch;
}


